@@ -21,10 +21,14 @@
     converted via Munch.to/fromDict().
 """
 
-__version__ = '2.2.0'
+__version__ = '2.3.2'
 VERSION = tuple(map(int, __version__.split('.')))
 
-__all__ = ('Munch', 'munchify', 'DefaultMunch', 'unmunchify')
+__all__ = ('Munch', 'munchify', 'DefaultMunch', 'DefaultFactoryMunch', 'unmunchify')
+
+
+from collections import defaultdict
+
 
 from .python3_compat import *   # pylint: disable=wildcard-import
 
@@ -163,6 +167,10 @@ class Munch(dict):
         """
         return unmunchify(self)
 
+    @property
+    def __dict__(self):
+        return self.toDict()
+
     def __repr__(self):
         """ Invertible* string-form of a Munch.
 
@@ -185,6 +193,21 @@ class Munch(dict):
     def __dir__(self):
         return list(iterkeys(self))
 
+    def __getstate__(self):
+        """ Implement a serializable interface used for pickling.
+
+        See https://docs.python.org/3.6/library/pickle.html.
+        """
+        return {k: v for k, v in self.items()}
+
+    def __setstate__(self, state):
+        """ Implement a serializable interface used for pickling.
+
+        See https://docs.python.org/3.6/library/pickle.html.
+        """
+        self.clear()
+        self.update(state)
+
     __members__ = __dir__  # for python2.x compatibility
 
     @classmethod
@@ -203,6 +226,16 @@ class Munch(dict):
         return type(self).fromDict(self)
 
 
+class AutoMunch(Munch):
+    def __setattr__(self, k, v):
+        """ Works the same as Munch.__setattr__ but if you supply
+            a dictionary as value it will convert it to another Munch.
+        """
+        if isinstance(v, dict) and not isinstance(v, (AutoMunch, Munch)):
+            v = munchify(v, AutoMunch)
+        super(AutoMunch, self).__setattr__(k, v)
+
+
 class DefaultMunch(Munch):
     """
     A Munch that returns a user-specified value for missing keys.
@@ -242,6 +275,23 @@ class DefaultMunch(Munch):
         except KeyError:
             return self.__default__
 
+    def __getstate__(self):
+        """ Implement a serializable interface used for pickling.
+
+        See https://docs.python.org/3.6/library/pickle.html.
+        """
+        return (self.__default__, {k: v for k, v in self.items()})
+
+    def __setstate__(self, state):
+        """ Implement a serializable interface used for pickling.
+
+        See https://docs.python.org/3.6/library/pickle.html.
+        """
+        self.clear()
+        default, state_dict = state
+        self.update(state_dict)
+        self.__default__ = default
+
     @classmethod
     def fromDict(cls, d, default=None):
         # pylint: disable=arguments-differ
@@ -255,6 +305,38 @@ class DefaultMunch(Munch):
             type(self).__name__, self.__undefined__, dict.__repr__(self))
 
 
+class DefaultFactoryMunch(defaultdict, Munch):
+    """ A Munch that calls a user-specified function to generate values for
+        missing keys like collections.defaultdict.
+
+        >>> b = DefaultFactoryMunch(list, {'hello': 'world!'})
+        >>> b.hello
+        'world!'
+        >>> b.foo
+        []
+        >>> b.bar.append('hello')
+        >>> b.bar
+        ['hello']
+    """
+
+    def __init__(self, default_factory, *args, **kwargs):
+        # pylint: disable=useless-super-delegation
+        super(DefaultFactoryMunch, self).__init__(default_factory, *args, **kwargs)
+
+    @classmethod
+    def fromDict(cls, d, default_factory):
+        # pylint: disable=arguments-differ
+        return munchify(d, factory=lambda d_: cls(default_factory, d_))
+
+    def copy(self):
+        return type(self).fromDict(self, default_factory=self.default_factory)
+
+    def __repr__(self):
+        factory = self.default_factory.__name__
+        return '{0}({1}, {2})'.format(
+            type(self).__name__, factory, dict.__repr__(self))
+
+
 # While we could convert abstract types like Mapping or Iterable, I think
 # munchify is more likely to "do what you mean" if it is conservative about
 # casting (ex: isinstance(str,Iterable) == True ).
